/**
 * @file  rcGetMiscSvrInfo.c
 *
 */

/*** Copyright (c), The Regents of the University of California            ***
 *** For more information please refer to files in the COPYRIGHT directory ***/
/* rcGetMiscSvrInfo.c - Client API call for GetMiscSvrInfo. Part of the 
 * reoutine may be generated by a script
 */
#include "getMiscSvrInfo.h"

/**
 * \fn rcGetMiscSvrInfo (rcComm_t *conn, miscSvrInfo_t **outSvrInfo)
 *
 * \brief Get the system information of the connected server.
 *
 * \user client
 *
 * \category misc operations
 *
 * \since 1.0
 *
 * \author  Mike Wan
 * \date    2007
 *
 * \remark none
 *
 * \note none
 *
 * \usage
 * Get the server info
 * \n miscSvrInfo_t *outSvrInfo = NULL;
 * \n status = rcGetMiscSvrInfo (conn, &outSvrInfo);
 * \n if (status < 0) {
 * \n .... handle the error
 * \n }
 * \n if (miscSvrInfo->serverType==RCAT_NOT_ENABLED) {
 * \n     printf("RCAT_NOT_ENABLED\n");
 * \n } else if (miscSvrInfo->serverType==RCAT_ENABLED) {
 * \n    printf("RCAT_ENABLED\n");
 * \n }
 * \n printf("relVersion=%s\n",miscSvrInfo->relVersion);
 * \n printf("apiVersion=%s\n",miscSvrInfo->apiVersion);
 * \n printf("rodsZone=%s\n",miscSvrInfo->rodsZone);
 *
 * \param[in] conn - A rcComm_t connection handle to the server.
 * \param[out] outSvrInfo - a miscSvrInfo_t structure containing the output.
 *
 * \return integer
 * \retval 0 on success
 * \sideeffect none
 * \pre none
 * \post none
 * \sa none
 * \bug  no known bugs
**/

int
rcGetMiscSvrInfo (rcComm_t *conn, miscSvrInfo_t **outSvrInfo)
{
    int status;

    *outSvrInfo = NULL;

    status = procApiRequest (conn, GET_MISC_SVR_INFO_AN, NULL, NULL,
      (void **) outSvrInfo, NULL);

    return (status);
}

